@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2021-2024. All Rights Reserved.
+ * Copyright Ericsson AB 2021-2026. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -55,12 +55,15 @@ BeamAssemblerCommon::BeamAssemblerCommon(BaseAssembler &assembler_)
                           8);
     ERTS_ASSERT(!err && "Failed to create .rodata section");
 
+    code.setErrorHandler(this);
+}
+
+void BeamAssemblerCommon::lateInit() {
 #ifdef DEBUG
     assembler.addDiagnosticOptions(DiagnosticOptions::kValidateAssembler);
 #endif
     assembler.addEncodingOptions(EncodingOptions::kOptimizeForSize |
                                  EncodingOptions::kOptimizedAlign);
-    code.setErrorHandler(this);
 }
 
 BeamAssemblerCommon::~BeamAssemblerCommon() {
@@ -1235,7 +1238,11 @@ void beam_jit_timeout_locked(Process *c_p) {
 void beam_jit_return_to_trace(Process *c_p,
                               Eterm session_weak_id,
                               Eterm *frame) {
-    if (ERTS_IS_P_TRACED_FL(c_p, F_TRACE_RETURN_TO)) {
+    ErtsTracerRef *ref =
+            get_tracer_ref_from_weak_id(&c_p->common, session_weak_id);
+
+    if (!ERTS_IS_PROC_SENSITIVE(c_p) && ref &&
+        IS_SESSION_TRACED_FL(ref, F_TRACE_RETURN_TO)) {
         ErtsCodePtr return_to_address;
         Uint *cpp;
 
@@ -1257,7 +1264,7 @@ void beam_jit_return_to_trace(Process *c_p,
         }
 
         ERTS_UNREQ_PROC_MAIN_LOCK(c_p);
-        erts_trace_return_to(c_p, return_to_address, session_weak_id);
+        erts_trace_return_to(c_p, return_to_address, ref);
         ERTS_REQ_PROC_MAIN_LOCK(c_p);
     }
 }
@@ -1280,8 +1287,8 @@ Eterm beam_jit_build_argument_list(Process *c_p, const Eterm *regs, int arity) {
 Export *beam_jit_handle_unloaded_fun(Process *c_p,
                                      Eterm *reg,
                                      int arity,
-                                     Eterm fun_thing) {
-    ErtsCodeIndex code_ix = erts_active_code_ix();
+                                     Eterm fun_thing,
+                                     ErtsCodeIndex code_ix) {
     Eterm module, args;
     ErlFunThing *funp;
     ErlFunEntry *fe;
@@ -1365,3 +1372,29 @@ bool beam_jit_is_shallow_boxed(Eterm term) {
         return false;
     }
 }
+
+#ifdef DEBUG
+void beam_jit_invalid_heap_ptr(Process *p, Eterm term) {
+    ASSERT((void *)term <= (void *)p->heap || (void *)term >= (void *)p->hend);
+
+    erts_fprintf(stderr,
+                 "term:       %p\n"
+                 "c_p:        %p\n"
+                 "heap:       %p\n"
+                 "high_water: %p\n"
+                 "hend:       %p\n"
+                 "abandoned:  %p\n",
+                 (void *)term,
+                 p,
+                 p->heap,
+                 p->high_water,
+                 p->hend,
+                 p->abandoned_heap);
+    if (p->old_heap != NULL && p->old_hend != NULL &&
+        (void *)term < (void *)p->old_hend &&
+        (void *)term >= (void *)p->old_heap) {
+        erts_fprintf(stderr, "the term is on the old heap\n");
+    }
+    abort();
+}
+#endif
