@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2008-2016. All Rights Reserved.
+%% Copyright Ericsson AB 2008-2026. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -29,7 +29,10 @@
 	 init_per_suite/1, end_per_suite/1,
 	 init_per_testcase/2, end_per_testcase/2]).
 
--compile(export_all).
+-export([calendarCtrl/1, treeCtrl/1, notebook/1, staticBoxSizer/1,
+         clipboard/1, helpFrame/1, htmlWindow/1, listCtrlSort/1, listCtrlVirtual/1,
+         radioBox/1, systemSettings/1, taskBarIcon/1, toolbar/1, popup/1, modal/1,
+         textCtrl/1, locale/1]).
 
 -include("wx_test_lib.hrl").
 
@@ -51,7 +54,8 @@ suite() -> [{ct_hooks,[ts_install_cth]}, {timetrap,{minutes,2}}].
 all() ->
     [calendarCtrl, treeCtrl, notebook, staticBoxSizer,
      clipboard, helpFrame, htmlWindow, listCtrlSort, listCtrlVirtual,
-     radioBox, systemSettings, taskBarIcon, toolbar, popup, modal].
+     radioBox, systemSettings, taskBarIcon, toolbar, popup, modal,
+     textCtrl, locale].
 
 groups() ->
     [].
@@ -83,7 +87,7 @@ calendarCtrl(Config) ->
 	true ->
 	    ?log("DateAttr is null~n",[]);
 	false ->
-	    ?log("DateAttr is useable~n",[]),
+	    ?log("DateAttr is usable~n",[]),
 	    DateAttr = ?mt(wxCalendarDateAttr, wxCalendarDateAttr:new()),
 	    wxCalendarDateAttr:setBackgroundColour(DateAttr, {0,243,0}),
 	    wxCalendarCtrl:setAttr(Cal, Day, DateAttr),
@@ -144,6 +148,9 @@ treeCtrl(Config) ->
     ?m({0, _}, wxTreeCtrl:hitTest(Tree, {X0+W0+W0, Y0+H0+4*H0})),
     ?m(false, wxTreeCtrl:isTreeItemIdOk(0)),
 
+    {N, Sels} = wxTreeCtrl:getSelections(Tree),
+    N = length(Sels),
+
     wxFrame:connect(Tree, command_tree_item_expanded),
     wxFrame:connect(Tree, command_tree_item_collapsed),
     wxFrame:connect(Frame, close_window),
@@ -216,7 +223,6 @@ notebook(Config) ->
 staticBoxSizer(TestInfo) when is_atom(TestInfo) -> wx_test_lib:tc_info(TestInfo);
 staticBoxSizer(Config) ->
     Wx = wx:new(),
-    wx:debug(2),
     Frame = wxFrame:new(Wx, ?wxID_ANY, "Frame"),
     Panel = wxPanel:new(Frame, []),
     InclSizer = ?mt(wxStaticBoxSizer,
@@ -241,6 +247,8 @@ clipboard(Config) ->
     wxTextCtrl:connect(Ctrl, command_text_paste, [{skip, true}]),
     wxWindow:show(Frame),
 
+    BlockWxDialogs = wxLogNull:new(),
+
     CB = ?mt(wxClipboard, wxClipboard:get()),
     wxClipboard:usePrimarySelection(CB),
     ?m(false, wx:is_null(CB)),
@@ -268,7 +276,10 @@ clipboard(Config) ->
 	    Paste = ?mt(wxTextDataObject, wxTextDataObject:new([{text,"From Erlang"}])),
 	    case wxClipboard:addData(CB,Paste) of
 		true ->
-		    ?log("Put text on clipboard~n", []);
+		    ?log("Put text on clipboard~n", []),
+                    ?log("Flushing ~n",[]),
+                    wxClipboard:flush(CB),
+                    ?log("Stopping ~n",[]);
 		false ->
 		    ?log("Couldn't copy data to clipboard~n",[])
 	    end,
@@ -276,9 +287,7 @@ clipboard(Config) ->
 	false ->
 	    ?log("Clipboard open failed~n",[])
     end,
-    ?log("Flushing ~n",[]),
-    wxClipboard:flush(CB),
-    ?log("Stopping ~n",[]),
+    wxLogNull:destroy(BlockWxDialogs),
     wx_test_lib:wx_destroy(Frame,Config).
 
 
@@ -370,7 +379,7 @@ listCtrlSort(Config) ->
     wx:foreach(Add, lists:seq(0,50)),
     wxWindow:show(Frame),
 
-    timer:sleep(2000),
+    timer:sleep(1000),
 
     Sort = fun() ->
 		   wxListCtrl:sortItems(LC, fun(A, B) ->
@@ -415,8 +424,15 @@ listCtrlVirtual(Config) ->
     Frame = wxFrame:new(Wx, ?wxID_ANY, "Frame"),
     IA = wxListItemAttr:new(),
     wxListItemAttr:setTextColour(IA, {190, 25, 25}),
+    Style = ?wxLC_SINGLE_SEL bor ?wxLC_REPORT bor ?wxLC_VIRTUAL bor
+        ?wxLC_HRULES bor ?wxHSCROLL bor ?wxVSCROLL,
+    %% Assert signed consts (wxVSCROLL)
+    Style = wxe_util:get_const(wxLC_SINGLE_SEL) bor wxe_util:get_const(wxLC_REPORT) bor
+        wxe_util:get_const(wxLC_VIRTUAL) bor wxe_util:get_const(wxLC_HRULES) bor wxe_util:get_const(wxHSCROLL)
+        bor wxe_util:get_const(wxVSCROLL),
+
     LC = wxListCtrl:new(Frame,
-			[{style, ?wxLC_REPORT bor ?wxLC_VIRTUAL},
+			[{style, Style},
 			 {onGetItemText, fun(_This, Item, 0) ->
 						 "Row " ++ integer_to_list(Item);
 					    (_, Item, 1) when Item rem 5 == 0 ->
@@ -614,7 +630,7 @@ lang_env() ->
     Env0 = os:getenv(),
     Env = [[R,"\n"]||R <- Env0],
     %%io:format("~p~n",[lists:sort(Env)]),
-    Opts = [global, multiline, {capture, all, list}],
+    Opts = [global, multiline, {capture, all, list}, unicode],
     format_env(re:run(Env, "LC_ALL.*", Opts)),
     format_env(re:run(Env, "^LANG.*=.*$", Opts)),
     ok.
@@ -625,66 +641,52 @@ format_env(nomatch) -> ok.
 %%  Add a testcase that tests that we can recurse in showModal
 %%  because it hangs in observer if object are not destroyed correctly
 %%  when popping the stack
-
 modal(Config) ->
     Wx = wx:new(),
-    case {?wxMAJOR_VERSION, ?wxMINOR_VERSION, ?wxRELEASE_NUMBER} of
-	{2, Min, Rel} when Min < 8 orelse (Min =:= 8 andalso Rel < 11) ->
-	    {skip, "old wxWidgets version"};
-	_ ->
-	    Frame = wxFrame:new(Wx, -1, "Test Modal windows"),
-	    wxFrame:show(Frame),
-	    Env = wx:get_env(),
-	    Tester = self(),
-	    ets:new(test_state, [named_table, public]),
-	    Upd = wxUpdateUIEvent:getUpdateInterval(),
-	    wxUpdateUIEvent:setUpdateInterval(500),
-	    _Pid = spawn(fun() ->
-				 wx:set_env(Env),
-				 modal_dialog(Frame, 1, Tester)
-			 end),
-	    %% need to sleep so we know that the window is stuck in
-	    %% the ShowModal event loop and not in an earlier event loop
-	    %% wx2.8 invokes the event loop from more calls than wx-3
-	    M1 = receive {dialog, W1, 1} -> timer:sleep(1200), ets:insert(test_state, {W1, ready}), W1 end,
-	    M2 = receive {dialog, W2, 2} -> timer:sleep(1200), ets:insert(test_state, {W2, ready}), W2 end,
+    Frame = wxFrame:new(Wx, -1, "Test Modal windows"),
+    wxFrame:show(Frame),
+    Env = wx:get_env(),
+    Upd = wxUpdateUIEvent:getUpdateInterval(),
+    wxUpdateUIEvent:setUpdateInterval(200),
+    Tester = spawn_link(fun() -> modal_test(Env) end),
+    D1 = wxTextEntryDialog:new(Frame, "Dialog 1"),
+    ShowCB = fun(#wx{event=Ev},_) ->
+                     case Ev of
+                         #wxShow{show=true} ->
+                             Tester ! {dialog, self(), D1},
+                             receive continue ->
+                                     D2 = wxTextEntryDialog:new(Frame, "Dialog 2"),
+                                     Tester ! {dialog, self(), D2},
+                                     Tester ! {dialog_done, self(), wxDialog:showModal(D2)}
+                             end;
+                         _ ->
+                             ignore
+                     end
+             end,
+    wxDialog:connect(D1, show, [{callback, ShowCB}]),
+    ?wxID_OK = wxDialog:showModal(D1),
+    wxUpdateUIEvent:setUpdateInterval(Upd),
+    wx_test_lib:wx_destroy(Frame,Config).
 
-	    receive done -> ok end,
-	    receive {dialog_done, M2, 2} -> M2 end,
-	    receive {dialog_done, M1, 1} -> M1 end,
-
-	    wxUpdateUIEvent:setUpdateInterval(Upd),
-	    wx_test_lib:wx_destroy(Frame,Config)
+%%
+%%  Add sleep before continuing so we recurse from
+%%  wxe_impl::dispatch_cb() to wxe_impl::dispatch() (with idle commands)
+%%  and handle events there so the queue should be reset and started from 0.
+%%
+modal_test(Env) ->
+    wx:set_env(Env),
+    receive
+        {dialog, CBpid, D1} ->
+            timer:sleep(500),
+            CBpid ! continue,
+            ?wxID_CANCEL = modal_test2(CBpid),
+            wxDialog:endModal(D1, ?wxID_OK)
     end.
 
-modal_dialog(Parent, Level, Tester) when Level < 3 ->
-    M1 = wxTextEntryDialog:new(Parent, "Dialog " ++ integer_to_list(Level)),
-    io:format("Creating dialog ~p ~p~n",[Level, M1]),
-    wxDialog:connect(M1, show, [{callback, fun(#wx{event=Ev},_) ->
-						   case Ev of
-						       #wxShow{show=true} ->
-							   Tester ! {dialog, M1, Level};
-						       _ -> ignore
-						   end
-					   end}]),
-    DoOnce = fun(_,_) ->
-		     case ets:take(test_state, M1) of
-			 [] -> ignore;
-			 [_] -> modal_dialog(M1, Level+1, Tester)
-		     end
-	     end,
-    wxDialog:connect(M1, update_ui, [{callback, DoOnce}]),
-    ?wxID_OK = wxDialog:showModal(M1),
-    wxDialog:destroy(M1),
-    case Level > 1 of
-	true ->
-	    io:format("~p: End dialog ~p ~p~n",[?LINE, Level-1, Parent]),
-	    wxDialog:endModal(Parent, ?wxID_OK);
-	false -> ok
-    end,
-    Tester ! {dialog_done, M1, Level},
-    ok;
-modal_dialog(Parent, Level, Tester) ->
-    io:format("~p: End dialog ~p ~p~n",[?LINE, Level-1, Parent]),
-    wxDialog:endModal(Parent, ?wxID_OK),
-    Tester ! done.
+modal_test2(CBpid) ->
+    receive
+        {dialog, CBpid, D2} ->
+            timer:sleep(500),
+            wxDialog:endModal(D2, ?wxID_CANCEL),
+            receive {dialog_done, CBpid, Res} -> Res end
+    end.
